@@ -31,11 +31,7 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-string.h>
 #include <eel/eel-vfs-extensions.h>
-#include <gtk/gtkaccellabel.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkimagemenuitem.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkiconfactory.h>
+#include <gtk/gtk.h>
 #include <libgnome/gnome-macros.h>
 #include <libgnome/gnome-util.h>
 #include <gio/gio.h>
@@ -329,7 +325,7 @@ nautilus_bookmark_icon_is_different (NautilusBookmark *bookmark,
 	g_assert (new_icon != NULL);
 
 	if (bookmark->details->icon == NULL) {
-		return new_icon == NULL;
+		return TRUE;
 	}
 	
 	return !g_icon_equal (bookmark->details->icon, new_icon) != 0;
@@ -350,7 +346,8 @@ nautilus_bookmark_update_icon (NautilusBookmark *bookmark)
 		return FALSE;
 	}
 
-	if (nautilus_file_check_if_ready (bookmark->details->file,
+	if (!nautilus_file_is_not_yet_confirmed (bookmark->details->file) &&
+	    nautilus_file_check_if_ready (bookmark->details->file,
 					  NAUTILUS_FILE_ATTRIBUTES_FOR_ICON)) {
 		new_icon = nautilus_file_get_gicon (bookmark->details->file, 0);
 		if (nautilus_bookmark_icon_is_different (bookmark, new_icon)) {
