@@ -25,7 +25,10 @@
 #ifndef NAUTILUS_FILE_UTILITIES_H
 #define NAUTILUS_FILE_UTILITIES_H
 
-#include <libgnomevfs/gnome-vfs-types.h>
+#include <gio/gio.h>
+
+#define NAUTILUS_SAVED_SEARCH_EXTENSION ".savedSearch"
+#define NAUTILUS_SAVED_SEARCH_MIMETYPE "application/x-gnome-saved-search"
 
 /* Recognizing special file names. */
 gboolean nautilus_file_name_matches_hidden_pattern   (const char *name_or_relative_uri);
@@ -34,21 +37,31 @@ gboolean nautilus_file_name_matches_backup_pattern   (const char *name_or_relati
 /* These functions all return something something that needs to be
  * freed with g_free, is not NULL, and is guaranteed to exist.
  */
+char *   nautilus_get_xdg_dir                        (const char *type);
 char *   nautilus_get_user_directory                 (void);
 char *   nautilus_get_desktop_directory              (void);
+GFile *  nautilus_get_desktop_location               (void);
 char *   nautilus_get_desktop_directory_uri          (void);
-gboolean nautilus_is_desktop_directory_file_escaped  (char *escaped_dirname,
-						      char *escaped_filename);
-gboolean nautilus_is_desktop_directory_escaped       (char *escaped_dir);
-gboolean nautilus_is_home_directory_file_escaped     (char *escaped_dirname,
-						      char *escaped_file);
+char *   nautilus_get_home_directory_uri             (void);
+gboolean nautilus_is_desktop_directory_file          (GFile *dir,
+						      const char *filename);
+gboolean nautilus_is_root_directory                  (GFile *dir);
+gboolean nautilus_is_desktop_directory               (GFile *dir);
+gboolean nautilus_is_home_directory                  (GFile *dir);
+gboolean nautilus_is_home_directory_file             (GFile *dir,
+						      const char *filename);
 char *   nautilus_get_gmc_desktop_directory          (void);
 char *   nautilus_get_pixmap_directory               (void);
 
+gboolean nautilus_should_use_templates_directory     (void);
 char *   nautilus_get_templates_directory            (void);
 char *   nautilus_get_templates_directory_uri        (void);
 void     nautilus_create_templates_directory         (void);
 
+char *   nautilus_get_searches_directory             (void);
+
+char *	 nautilus_compute_title_for_location	     (GFile *file);
+
 /* This function returns something that needs to be freed with g_free,
  * is not NULL, but is not garaunteed to exist */
 char *   nautilus_get_desktop_directory_uri_no_create (void);
@@ -66,6 +79,8 @@ char *   nautilus_pixmap_file                        (const char *partial_path);
 /* Locate a file in either the uers directory or the datadir. */
 char *   nautilus_get_data_file_path                 (const char *partial_path);
 
+gboolean nautilus_is_file_roller_installed           (void);
+
 /* Return an allocated file name that is guranteed to be unique, but
  * tries to make the name readable to users.
  * This isn't race-free, so don't use for security-related things
@@ -74,10 +89,10 @@ char *   nautilus_ensure_unique_file_name            (const char *directory_uri,
 						      const char *base_name,
 			                              const char *extension);
 char *   nautilus_unique_temporary_file_name         (void);
-char *   nautilus_find_file_in_gnome_path            (char       *file);
-GList *  nautilus_find_all_files_in_gnome_path       (char       *file);
 
-const char *nautilus_get_vfs_method_display_name (char *method);
-char *      nautilus_get_uri_shortname_for_display (GnomeVFSURI *uri);
+GFile *  nautilus_find_existing_uri_in_hierarchy     (GFile *location);
+
+GFile *
+nautilus_find_file_insensitive (GFile *parent, const gchar *name);
 
 #endif /* NAUTILUS_FILE_UTILITIES_H */
