@@ -30,7 +30,7 @@
 #include "nautilus-file-utilities.h"
 #include "nautilus-search-engine.h"
 #include <eel/eel-glib-extensions.h>
-#include <gtk/gtksignal.h>
+#include <gtk/gtk.h>
 #include <gio/gio.h>
 #include <libgnome/gnome-macros.h>
 #include <string.h>
