@@ -29,7 +29,6 @@
 #include "nautilus-application.h"
 
 
-#include "file-manager/fm-ditem-page.h"
 #include "file-manager/fm-desktop-icon-view.h"
 #include "file-manager/fm-icon-view.h"
 #include "file-manager/fm-list-view.h"
@@ -48,7 +47,6 @@
 #include <fcntl.h>
 #include <string.h>
 #include "nautilus-desktop-window.h"
-#include "nautilus-first-time-druid.h"
 #include "nautilus-main.h"
 #include "nautilus-spatial-window.h"
 #include "nautilus-navigation-window.h"
@@ -68,7 +66,7 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
 #include <gdk/gdkx.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtk.h>
 #include <libgnome/gnome-config.h>
 #include <libgnomeui/gnome-authentication-manager.h>
 #include <libgnomeui/gnome-client.h>
@@ -127,8 +125,6 @@ static void     update_session                    (gpointer                  cal
 static void     init_session                      (void);
 static gboolean is_kdesktop_present               (void);
 
-static char    *save_session_to_file              (void);
-
 BONOBO_CLASS_BOILERPLATE (NautilusApplication, nautilus_application,
 			  BonoboGenericFactory, BONOBO_TYPE_GENERIC_FACTORY)
 
@@ -221,6 +217,7 @@ nautilus_application_instance_init (NautilusApplication *application)
 	fm_icon_view_register ();
 	fm_desktop_icon_view_register ();
 	fm_list_view_register ();
+	fm_compact_view_register ();
 #if ENABLE_EMPTY_VIEW
 	fm_empty_view_register ();
 #endif /* ENABLE_EMPTY_VIEW */
@@ -424,8 +421,6 @@ finish_startup (NautilusApplication *application)
 	/* initialize nautilus modules */
 	nautilus_module_init ();
 
-	nautilus_module_add_type (FM_TYPE_DITEM_PAGE);
-
 	/* attach menu-provider module callback */
 	menu_provider_init_callback ();
 	
@@ -540,7 +535,6 @@ nautilus_application_startup (NautilusApplication *application,
 			      gboolean restart_shell,
 			      gboolean no_default_window,
 			      gboolean no_desktop,
-			      gboolean do_first_time_druid_check,
 			      gboolean browser_window,
 			      const char *startup_id,
 			      const char *geometry,
@@ -600,18 +594,18 @@ nautilus_application_startup (NautilusApplication *application,
 			 * this problem but we don't exactly understand why,
 			 * since neither of the above causes explain it.
 			 */
-			message = _("Nautilus can't be used now. "
+			message = _("Nautilus cannot be used now. "
 				    "Running the command \"bonobo-slay\""
 				    " from the console may fix the problem. If not,"
 				    " you can try rebooting the computer or"
 				    " installing Nautilus again.");
 			/* FIXME bugzilla.gnome.org 42536: The guesses and stuff here are lame. */
-			detailed_message = _("Nautilus can't be used now. "
+			detailed_message = _("Nautilus cannot be used now. "
 					     "Running the command \"bonobo-slay\" "
 					     "from the console may fix the problem. If not, "
 					     "you can try rebooting the computer or "
 					     "installing Nautilus again.\n\n"
-					     "Bonobo couldn't locate the Nautilus_shell.server file. "
+					     "Bonobo could not locate the Nautilus_shell.server file. "
 					     "One cause of this seems to be an LD_LIBRARY_PATH "
 					     "that does not include the bonobo-activation library's directory. "
 					     "Another possible cause would be bad install "
@@ -620,7 +614,7 @@ nautilus_application_startup (NautilusApplication *application,
 					     "Bonobo Activation and GConf processes, which may be needed by "
 					     "other applications.\n\n"
 					     "Sometimes killing bonobo-activation-server and gconfd fixes "
-					     "the problem, but we don't know why.\n\n"
+					     "the problem, but we do not know why.\n\n"
 					     "We have also seen this error when a faulty "
 					     "version of bonobo-activation was installed.");
 			break;
@@ -636,8 +630,8 @@ nautilus_application_startup (NautilusApplication *application,
 			 * current OAF. I guess I read the code wrong. Need to figure out when and make a
 			 * good message.
 			 */
-			message = _("Nautilus can't be used now, due to an unexpected error.");
-			detailed_message = _("Nautilus can't be used now, due to an unexpected error "
+			message = _("Nautilus cannot be used now, due to an unexpected error.");
+			detailed_message = _("Nautilus cannot be used now, due to an unexpected error "
 					     "from Bonobo when attempting to register the file manager view server.");
 			break;
 		}
@@ -654,15 +648,15 @@ nautilus_application_startup (NautilusApplication *application,
 			 */
 			if (result == Bonobo_ACTIVATION_REG_SUCCESS) {
 				/* FIXME bugzilla.gnome.org 42538: When can this happen? */
-				message = _("Nautilus can't be used now, due to an unexpected error.");
-				detailed_message = _("Nautilus can't be used now, due to an unexpected error "
+				message = _("Nautilus cannot be used now, due to an unexpected error.");
+				detailed_message = _("Nautilus cannot be used now, due to an unexpected error "
 						     "from Bonobo when attempting to locate the factory. "
 						     "Killing bonobo-activation-server and restarting Nautilus may help fix the problem.");
 			} else {
 				num_failures++;
 				if (num_failures > 20) {
-					message = _("Nautilus can't be used now, due to an unexpected error.");
-					detailed_message = _("Nautilus can't be used now, due to an unexpected error "
+					message = _("Nautilus cannot be used now, due to an unexpected error.");
+					detailed_message = _("Nautilus cannot be used now, due to an unexpected error "
 							     "from Bonobo when attempting to locate the shell object. "
 							     "Killing bonobo-activation-server and restarting Nautilus may help fix the problem.");
 					
@@ -895,18 +889,18 @@ static NautilusSpatialWindow *
 nautilus_application_get_existing_spatial_window (GFile *location)
 {
 	GList *l;
+	NautilusWindowSlot *slot;
 
 	for (l = nautilus_application_get_spatial_window_list ();
 	     l != NULL; l = l->next) {
 		GFile *window_location;
 
-		window_location = nautilus_window_get_location (NAUTILUS_WINDOW (l->data));
+		slot = NAUTILUS_WINDOW (l->data)->details->active_slot;
+		window_location = slot->location;
 		if (window_location != NULL) {
 			if (g_file_equal (location, window_location)) {
-				g_object_unref (window_location);
 				return NAUTILUS_SPATIAL_WINDOW (l->data);
 			}
-			g_object_unref (window_location);
 		}
 	}
 	return NULL;
@@ -917,14 +911,16 @@ find_parent_spatial_window (NautilusSpatialWindow *window)
 {
 	NautilusFile *file;
 	NautilusFile *parent_file;
+	NautilusWindowSlot *slot;
 	GFile *location;
 
-	location = nautilus_window_get_location (NAUTILUS_WINDOW (window));
+	slot = NAUTILUS_WINDOW (window)->details->active_slot;
+
+	location = slot->location;
 	if (location == NULL) {
 		return NULL;
 	}
 	file = nautilus_file_get (location);
-	g_object_unref (location);
 
 	if (!file) {
 		return NULL;
@@ -1166,13 +1162,15 @@ nautilus_application_present_spatial_window_with_selection (NautilusApplication
 	for (l = nautilus_application_get_spatial_window_list ();
 	     l != NULL; l = l->next) {
 		NautilusWindow *existing_window;
+		NautilusWindowSlot *slot;
 		GFile *existing_location;
-               	     
+
 		existing_window = NAUTILUS_WINDOW (l->data);
-		existing_location = existing_window->details->pending_location;
+		slot = existing_window->details->active_slot;
+		existing_location = slot->pending_location;
 		
 		if (existing_location == NULL) {
-			existing_location = existing_window->details->location;
+			existing_location = slot->location;
 		}
 		
 		if (g_file_equal (existing_location, location)) {
@@ -1183,8 +1181,8 @@ nautilus_application_present_spatial_window_with_selection (NautilusApplication
 
 			gtk_window_present (GTK_WINDOW (existing_window));
 			if (new_selection &&
-			    existing_window->content_view != NULL) {
-				nautilus_view_set_selection (existing_window->content_view, new_selection);
+			    slot->content_view != NULL) {
+				nautilus_view_set_selection (slot->content_view, new_selection);
 			}
 
 			uri = g_file_get_uri (location);
@@ -1347,6 +1345,12 @@ volume_added_callback (GVolumeMonitor *monitor,
 	    g_volume_should_automount (volume) &&
 	    g_volume_can_mount (volume)) {
 		nautilus_file_operations_mount_volume (NULL, volume, TRUE);
+	} else {
+		/* Allow nautilus_autorun() to run. When the mount is later
+		 * added programmatically (i.e. for a blank CD),
+		 * nautilus_autorun() will be called by mount_added_callback(). */
+		nautilus_allow_autorun_for_volume (volume);
+		nautilus_allow_autorun_for_volume_finish (volume);
 	}
 }
 
@@ -1456,6 +1460,7 @@ mount_removed_callback (GVolumeMonitor *monitor,
 {
 	GList *window_list, *node, *close_list;
 	NautilusWindow *window;
+	NautilusWindowSlot *slot;
 	GFile *root;
 
 	close_list = NULL;
@@ -1467,25 +1472,28 @@ mount_removed_callback (GVolumeMonitor *monitor,
 	/* Construct a list of windows to be closed. Do not add the non-closable windows to the list. */
 	for (node = window_list; node != NULL; node = node->next) {
 		window = NAUTILUS_WINDOW (node->data);
-		if (window != NULL && window_can_be_closed (window)) {
-			GFile *location;
-
-			location = nautilus_window_get_location (window);
-
-			if (g_file_equal(location, root) || g_file_has_prefix (location, root)) {
-				close_list = g_list_prepend (close_list, window);
-			} 
-			g_object_unref (location);
+  		if (window != NULL && window_can_be_closed (window)) {
+			GList *l;
+  			GFile *location;
+  
+			for (l = window->details->slots; l != NULL; l = l->next) {
+				slot = l->data;
+				location = slot->location;
+				if (g_file_has_prefix (location, root)) {
+					close_list = g_list_prepend (close_list, slot);
+				} 
+			}
 		}
 	}
 
 	/* Handle the windows in the close list. */
 	for (node = close_list; node != NULL; node = node->next) {
-		window = NAUTILUS_WINDOW (node->data);
+		slot = node->data;
+		window = slot->window;
 		if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-			nautilus_window_close (window);
+			nautilus_window_slot_close (slot);
 		} else {
-			nautilus_window_go_home (window);
+			nautilus_window_slot_go_home (slot, FALSE);
 		}
 	}
 
@@ -1537,8 +1545,8 @@ icon_from_string (const char *string)
 	return NULL;
 }
 
-static char *
-save_session_to_file (void)
+char *
+nautilus_application_save_session_to_file (void)
 {
 	xmlDocPtr doc;
 	xmlNodePtr root_node, history_node;
@@ -1592,8 +1600,10 @@ save_session_to_file (void)
 	}
 
 	for (l = nautilus_application_window_list; l != NULL; l = l->next) {
-		xmlNodePtr win_node;
+		xmlNodePtr win_node, slot_node;
 		NautilusWindow *window;
+		NautilusWindowSlot *slot, *active_slot;
+		GList *slots, *m;
 		char *tmp;
 
 		window = l->data;
@@ -1623,9 +1633,30 @@ save_session_to_file (void)
 			}
 		}
 
-		tmp = nautilus_window_get_location_uri (window);
+		slots = nautilus_window_get_slots (window);
+		active_slot = nautilus_window_get_active_slot (window);
+
+		/* store one slot as window location. Otherwise
+		 * older Nautilus versions will bail when reading the file. */
+		tmp = nautilus_window_slot_get_location_uri (active_slot);
 		xmlNewProp (win_node, "location", tmp);
 		g_free (tmp);
+
+		for (m = slots; m != NULL; m = m->next) {
+			slot = NAUTILUS_WINDOW_SLOT (m->data);
+
+			slot_node = xmlNewChild (win_node, NULL, "slot", NULL);
+
+			tmp = nautilus_window_slot_get_location_uri (slot);
+			xmlNewProp (slot_node, "location", tmp);
+			g_free (tmp);
+
+			if (slot == active_slot) {
+				xmlNewProp (slot_node, "active", "TRUE");
+			}
+		}
+
+		g_list_free (slots);
 	}
 
 	dir = nautilus_get_user_directory ();
@@ -1729,8 +1760,10 @@ nautilus_application_load_session (NautilusApplication *application,
 					}
 				} else if (!strcmp (node->name, "window")) {
 					NautilusWindow *window;
-					xmlChar *type, *location_uri;
+					xmlChar *type, *location_uri, *slot_uri;
+					xmlNodePtr slot_node;
 					GFile *location;
+					int i;
 
 					type = xmlGetProp (node, "type");
 					if (type == NULL) {
@@ -1781,9 +1814,37 @@ nautilus_application_load_session (NautilusApplication *application,
 							gtk_window_set_keep_above (GTK_WINDOW (window), FALSE);
 						}
 
-						location = g_file_new_for_uri (location_uri);
-						nautilus_window_open_location (window, location, FALSE);
-						g_object_unref (location);
+						for (i = 0, slot_node = node->children; slot_node != NULL; slot_node = slot_node->next) {
+							if (!strcmp (slot_node->name, "slot")) {
+								slot_uri = xmlGetProp (slot_node, "location");
+								if (slot_uri != NULL) {
+									NautilusWindowSlot *slot;
+
+									if (i == 0) {
+										slot = window->details->active_slot;
+									} else {
+										slot = nautilus_window_open_slot (window, NAUTILUS_WINDOW_OPEN_SLOT_APPEND);
+									}
+
+									location = g_file_new_for_uri (slot_uri);
+									nautilus_window_slot_open_location (slot, location, FALSE);
+
+									if (xmlHasProp (slot_node, "active")) {
+										nautilus_window_set_active_slot (window, slot);
+									}
+
+									i++;
+								}
+								xmlFree (slot_uri);
+							}
+						}
+
+						if (i == 0) {
+							/* This may be an old session file */
+							location = g_file_new_for_uri (location_uri);
+							nautilus_window_slot_open_location (window->details->active_slot, location, FALSE);
+							g_object_unref (location);
+						}
 					} else if (!strcmp (type, "spatial")) {
 						location = g_file_new_for_uri (location_uri);
 						window = nautilus_application_present_spatial_window (application, NULL, NULL, location, gdk_screen_get_default ());
@@ -1832,7 +1893,7 @@ save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style, gint s
 
 	argv[0] = "nautilus";
 
-	argv[2] = save_session_to_file ();
+	argv[2] = nautilus_application_save_session_to_file ();
 	if (argv[2] != NULL) {
 		argv[1] = "--load-session";
 	}
