@@ -42,29 +42,21 @@
 #include "nautilus-search-bar.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-zoom-control.h"
+#include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-string.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
-#include <gtk/gtkdnd.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkmenubar.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkuimanager.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <libgnome/gnome-macros.h>
-#include <libgnomeui/gnome-uidefs.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libnautilus-private/nautilus-dnd.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-ui-utilities.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-horizontal-splitter.h>
-#include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-metadata.h>
 #include <libnautilus-private/nautilus-mime-actions.h>
 #include <libnautilus-private/nautilus-program-choosing.h>
@@ -91,10 +83,6 @@ struct _NautilusSpatialWindowDetails {
 	GtkWidget *location_button;
 	GtkWidget *location_label;
 	GtkWidget *location_icon;
-
-	GtkWidget *query_editor;
-	
-	GnomeVFSURI *location;
 };
 
 static const GtkTargetEntry location_button_drag_types[] = {
@@ -102,17 +90,23 @@ static const GtkTargetEntry location_button_drag_types[] = {
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
 };
 
-GNOME_CLASS_BOILERPLATE (NautilusSpatialWindow, nautilus_spatial_window,
-			 NautilusWindow, NAUTILUS_TYPE_WINDOW)
+G_DEFINE_TYPE(NautilusSpatialWindow, nautilus_spatial_window, NAUTILUS_TYPE_WINDOW)
+#define parent_class nautilus_spatial_window_parent_class
+
+static void nautilus_spatial_window_save_geometry (NautilusWindowSlot *slot);
 
 static gboolean
 save_window_geometry_timeout (gpointer callback_data)
 {
 	NautilusSpatialWindow *window;
+	NautilusWindowSlot *slot;
 	
 	window = NAUTILUS_SPATIAL_WINDOW (callback_data);
-	
-	nautilus_spatial_window_save_geometry (window);
+	slot = nautilus_window_get_active_slot (NAUTILUS_WINDOW (window));
+
+	if (slot != NULL) {
+		nautilus_spatial_window_save_geometry (slot);
+	}
 
 	window->details->save_geometry_timeout_id = 0;
 
@@ -128,7 +122,7 @@ nautilus_spatial_window_configure_event (GtkWidget *widget,
 	
 	window = NAUTILUS_SPATIAL_WINDOW (widget);
 
-	GTK_WIDGET_CLASS (parent_class)->configure_event (widget, event);
+	GTK_WIDGET_CLASS (nautilus_spatial_window_parent_class)->configure_event (widget, event);
 	
 	/* Only save the geometry if the user hasn't resized the window
 	 * for a second. Otherwise delay the callback another second.
@@ -169,15 +163,20 @@ static void
 nautilus_spatial_window_unrealize (GtkWidget *widget)
 {
 	NautilusSpatialWindow *window;
+	NautilusWindowSlot *slot;
 	
 	window = NAUTILUS_SPATIAL_WINDOW (widget);
+	slot = nautilus_window_get_active_slot (NAUTILUS_WINDOW (window));
 
-	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+	GTK_WIDGET_CLASS (nautilus_spatial_window_parent_class)->unrealize (widget);
 
 	if (window->details->save_geometry_timeout_id != 0) {
 		g_source_remove (window->details->save_geometry_timeout_id);
 		window->details->save_geometry_timeout_id = 0;
-		nautilus_spatial_window_save_geometry (window);
+
+		if (slot != NULL) {
+			nautilus_spatial_window_save_geometry (slot);
+		}
 	}
 }
 
@@ -185,32 +184,44 @@ static gboolean
 nautilus_spatial_window_state_event (GtkWidget *widget,
 				     GdkEventWindowState *event)
 {
-	if (event->changed_mask & GDK_WINDOW_STATE_MAXIMIZED &&
-	    NAUTILUS_WINDOW (widget)->details->viewed_file != NULL) {
-		nautilus_file_set_boolean_metadata (NAUTILUS_WINDOW (widget)->details->viewed_file,
-						    NAUTILUS_METADATA_KEY_WINDOW_MAXIMIZED,
-						    FALSE,
-						    event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED);
-	}
+	NautilusWindow *window;
+	NautilusWindowSlot *slot;
+	NautilusFile *viewed_file;
 
-	if (event->changed_mask & GDK_WINDOW_STATE_STICKY &&
-	    NAUTILUS_WINDOW (widget)->details->viewed_file != NULL) {
-		nautilus_file_set_boolean_metadata (NAUTILUS_WINDOW (widget)->details->viewed_file,
-						    NAUTILUS_METADATA_KEY_WINDOW_STICKY,
-						    FALSE,
-						    event->new_window_state & GDK_WINDOW_STATE_STICKY);
-	}
+	window = NAUTILUS_WINDOW (widget);
+	slot = window->details->active_slot;
+	viewed_file = slot->viewed_file;
 
-	if (event->changed_mask & GDK_WINDOW_STATE_ABOVE &&
-	    NAUTILUS_WINDOW (widget)->details->viewed_file != NULL) {
-		nautilus_file_set_boolean_metadata (NAUTILUS_WINDOW (widget)->details->viewed_file,
-						    NAUTILUS_METADATA_KEY_WINDOW_KEEP_ABOVE,
-						    FALSE,
-						    event->new_window_state & GDK_WINDOW_STATE_ABOVE);
-	}
+	if (!NAUTILUS_IS_DESKTOP_WINDOW (widget)) {
+		
+		if (event->changed_mask & GDK_WINDOW_STATE_MAXIMIZED &&
+		    viewed_file != NULL) {
+			nautilus_file_set_boolean_metadata (viewed_file,
+							    NAUTILUS_METADATA_KEY_WINDOW_MAXIMIZED,
+							    FALSE,
+							    event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED);
+		}
+		
+		if (event->changed_mask & GDK_WINDOW_STATE_STICKY &&
+		    viewed_file != NULL) {
+			nautilus_file_set_boolean_metadata (viewed_file,
+							    NAUTILUS_METADATA_KEY_WINDOW_STICKY,
+							    FALSE,
+							    event->new_window_state & GDK_WINDOW_STATE_STICKY);
+		}
+		
+		if (event->changed_mask & GDK_WINDOW_STATE_ABOVE &&
+		    viewed_file != NULL) {
+			nautilus_file_set_boolean_metadata (viewed_file,
+							    NAUTILUS_METADATA_KEY_WINDOW_KEEP_ABOVE,
+							    FALSE,
+							    event->new_window_state & GDK_WINDOW_STATE_ABOVE);
+		}
 
-	if (GTK_WIDGET_CLASS (parent_class)->window_state_event != NULL) {
-		return GTK_WIDGET_CLASS (parent_class)->window_state_event (widget, event);
+	}
+	
+	if (GTK_WIDGET_CLASS (nautilus_spatial_window_parent_class)->window_state_event != NULL) {
+		return GTK_WIDGET_CLASS (nautilus_spatial_window_parent_class)->window_state_event (widget, event);
 	}
 
 	return FALSE;
@@ -225,7 +236,7 @@ nautilus_spatial_window_destroy (GtkObject *object)
 
 	window->details->content_box = NULL;
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	GTK_OBJECT_CLASS (nautilus_spatial_window_parent_class)->destroy (object);
 }
 
 static void
@@ -235,28 +246,25 @@ nautilus_spatial_window_finalize (GObject *object)
 	
 	window = NAUTILUS_SPATIAL_WINDOW (object);
 
-	if (window->details->location != NULL) {
-		gnome_vfs_uri_unref (window->details->location);
-	}
-
 	if (window->details->last_geometry != NULL) {
 		g_free (window->details->last_geometry);
 	}
 
-	g_free (window->details);
-	window->details = NULL;
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (nautilus_spatial_window_parent_class)->finalize (object);
 }
 
-void
-nautilus_spatial_window_save_geometry (NautilusSpatialWindow *window)
+static void
+nautilus_spatial_window_save_geometry (NautilusWindowSlot *slot)
 {
+	NautilusWindow *window;
+	NautilusFile *viewed_file;
 	char *geometry_string;
 
-	g_assert (NAUTILUS_IS_WINDOW (window));
+	window = NAUTILUS_WINDOW (slot->window);
 
-	if (NAUTILUS_WINDOW (window)->details->viewed_file == NULL) {
+	viewed_file = slot->viewed_file;
+
+	if (viewed_file == NULL) {
 		/* We never showed a file */
 		return;
 	}
@@ -265,7 +273,7 @@ nautilus_spatial_window_save_geometry (NautilusSpatialWindow *window)
 	    !(gdk_window_get_state (GTK_WIDGET(window)->window) & GDK_WINDOW_STATE_MAXIMIZED)) {
 		geometry_string = eel_gtk_window_get_geometry_string (GTK_WINDOW (window));
 		
-		nautilus_file_set_metadata (NAUTILUS_WINDOW (window)->details->viewed_file,
+		nautilus_file_set_metadata (viewed_file,
 					    NAUTILUS_METADATA_KEY_WINDOW_GEOMETRY,
 					    NULL,
 					    geometry_string);
@@ -274,32 +282,35 @@ nautilus_spatial_window_save_geometry (NautilusSpatialWindow *window)
 	}
 }
 
-void
-nautilus_spatial_window_save_scroll_position (NautilusSpatialWindow *window)
+static void
+nautilus_spatial_window_save_scroll_position (NautilusWindowSlot *slot)
 {
-	NautilusWindow *parent;
+	NautilusWindow *window;
 	char *scroll_string;
 
-	parent = NAUTILUS_WINDOW(window);
-	
-	if (parent->content_view == NULL) {
+	window = NAUTILUS_WINDOW (slot->window);
+
+	if (slot->content_view == NULL) {
 		return;
 	}
 	
-	scroll_string = nautilus_view_get_first_visible_file (parent->content_view);
-	nautilus_file_set_metadata (parent->details->viewed_file,
+	scroll_string = nautilus_view_get_first_visible_file (slot->content_view);
+	nautilus_file_set_metadata (slot->viewed_file,
 				    NAUTILUS_METADATA_KEY_WINDOW_SCROLL_POSITION,
 				    NULL,
 				    scroll_string);
 	g_free (scroll_string);
 }
 
-void
-nautilus_spatial_window_save_show_hidden_files_mode (NautilusSpatialWindow *window)
+static void
+nautilus_spatial_window_save_show_hidden_files_mode (NautilusWindowSlot *slot)
 {
+	NautilusWindow *window;
 	char *show_hidden_file_setting;
 	NautilusWindowShowHiddenFilesMode mode;
 
+	window = NAUTILUS_WINDOW (slot->window);
+
 	mode = NAUTILUS_WINDOW (window)->details->show_hidden_files_mode;
 	if (mode != NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT) {
 		if (mode == NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_ENABLE) {
@@ -307,7 +318,7 @@ nautilus_spatial_window_save_show_hidden_files_mode (NautilusSpatialWindow *wind
 		} else {
 			show_hidden_file_setting = "0";
 		}
-		nautilus_file_set_metadata (NAUTILUS_WINDOW (window)->details->viewed_file,
+		nautilus_file_set_metadata (slot->viewed_file,
 			 		    NAUTILUS_METADATA_KEY_WINDOW_SHOW_HIDDEN_FILES,
 				    	    NULL,
 				    	    show_hidden_file_setting);
@@ -317,15 +328,16 @@ nautilus_spatial_window_save_show_hidden_files_mode (NautilusSpatialWindow *wind
 static void
 nautilus_spatial_window_show (GtkWidget *widget)
 {	
-	NautilusSpatialWindow *window;
+	NautilusWindow *window;
+	NautilusWindowSlot *slot;
 
-	window = NAUTILUS_SPATIAL_WINDOW (widget);
-	
-	GTK_WIDGET_CLASS (parent_class)->show (widget);
+	window = NAUTILUS_WINDOW (widget);
+	slot = nautilus_window_get_active_slot (window);
 
-	if (NAUTILUS_WINDOW (window)->details->search_mode &&
-	    window->details->query_editor != NULL) {
-		nautilus_query_editor_grab_focus (NAUTILUS_QUERY_EDITOR (window->details->query_editor));
+	GTK_WIDGET_CLASS (nautilus_spatial_window_parent_class)->show (widget);
+
+	if (slot != NULL && slot->query_editor != NULL) {
+		nautilus_query_editor_grab_focus (NAUTILUS_QUERY_EDITOR (slot->query_editor));
 	}
 }
 
@@ -358,108 +370,39 @@ real_prompt_for_location (NautilusWindow *window,
 	gtk_widget_show (dialog);
 }
 
-static void
-query_editor_changed_callback (NautilusSearchBar *bar,
-			       NautilusQuery *query,
-			       gboolean reload,
-			       NautilusWindow *window)
+static NautilusIconInfo *
+real_get_icon (NautilusWindow *window,
+	       NautilusWindowSlot *slot)
 {
-	NautilusDirectory *directory;
-
-	directory = nautilus_directory_get_for_file (window->details->viewed_file);
-	g_assert (NAUTILUS_IS_SEARCH_DIRECTORY (directory));
-
-	nautilus_search_directory_set_query (NAUTILUS_SEARCH_DIRECTORY (directory),
-					     query);
-	if (reload) {
-		nautilus_window_reload (window);
-	}
-
-	nautilus_directory_unref (directory);
+	return nautilus_file_get_icon (slot->viewed_file, 48,
+				       NAUTILUS_FILE_ICON_FLAGS_IGNORE_VISITING);
 }
 
 static void
-real_set_search_mode (NautilusWindow *window, gboolean search_mode,
-		      NautilusSearchDirectory *search_directory)
+sync_window_title (NautilusWindow *window)
 {
-	NautilusSpatialWindow *spatial_window;
-	GtkWidget *query_editor;
-	NautilusQuery *query;
+	NautilusWindowSlot *slot;
 
-	spatial_window = NAUTILUS_SPATIAL_WINDOW (window);
+	slot = nautilus_window_get_active_slot (window);
 
-	spatial_window->details->query_editor = NULL;
-	
-	if (search_mode) {
-		query_editor = nautilus_query_editor_new (nautilus_search_directory_is_saved_search (search_directory),
-							  nautilus_search_directory_is_indexed (search_directory));
-		spatial_window->details->query_editor = query_editor;
-		
-		nautilus_window_add_extra_location_widget (window, query_editor);
-		gtk_widget_show (query_editor);
-		nautilus_query_editor_grab_focus (NAUTILUS_QUERY_EDITOR (query_editor));
-		g_signal_connect_object (query_editor, "changed",
-					 G_CALLBACK (query_editor_changed_callback), window, 0);
-		
-		query = nautilus_search_directory_get_query (search_directory);
-		if (query != NULL) {
-			nautilus_query_editor_set_query (NAUTILUS_QUERY_EDITOR (query_editor),
-							 query);
-			g_object_unref (query);
-		} else {
-			nautilus_query_editor_set_default_query (NAUTILUS_QUERY_EDITOR (query_editor));
-		}
-	} 
-}
-
-static char *
-real_get_icon_name (NautilusWindow *window)
-{
-	return nautilus_icon_factory_get_icon_for_file (window->details->viewed_file, FALSE);
-}
-
-static gboolean
-real_set_title (NautilusWindow *window, const char *title)
-{
-	gboolean changed;
-
-	changed = EEL_CALL_PARENT_WITH_RETURN_VALUE
-		(NAUTILUS_WINDOW_CLASS, set_title, (window, title));
-
-	if (changed && title[0] == '\0') {
+	if (slot->title == NULL || slot->title[0] == '\0') {
 		gtk_window_set_title (GTK_WINDOW (window), _("Nautilus"));
-	} else if (changed) {
+	} else {
 		char *window_title;
 
-		window_title = eel_str_middle_truncate (title, MAX_TITLE_LENGTH);
+		window_title = eel_str_middle_truncate (slot->title, MAX_TITLE_LENGTH);
 		gtk_window_set_title (GTK_WINDOW (window), window_title);
 		g_free (window_title);
 	}
-
-	return changed;
 }
 
 static void
-real_set_content_view_widget (NautilusWindow *window,
-			      NautilusView *new_view)
+real_sync_title (NautilusWindow *window,
+		 NautilusWindowSlot *slot)
 {
-	GtkWidget *widget;
-	
-	EEL_CALL_PARENT (NAUTILUS_WINDOW_CLASS, set_content_view_widget,
-			 (window, new_view));
+	g_assert (slot == nautilus_window_get_active_slot (window));
 
-	widget = nautilus_view_get_widget (new_view);
-	
-	gtk_container_add (GTK_CONTAINER (NAUTILUS_SPATIAL_WINDOW (window)->details->content_box),
-			   widget);
-}
-
-static void
-real_window_close (NautilusWindow *window)
-{
-	nautilus_spatial_window_save_geometry (NAUTILUS_SPATIAL_WINDOW (window));
-	nautilus_spatial_window_save_scroll_position (NAUTILUS_SPATIAL_WINDOW (window));
-	nautilus_spatial_window_save_show_hidden_files_mode (NAUTILUS_SPATIAL_WINDOW (window));
+	sync_window_title (window);
 }
 
 static void 
@@ -474,9 +417,9 @@ real_get_default_size (NautilusWindow *window,
 	}
 }
 
-
 static void
-real_set_throbber_active (NautilusWindow *window, gboolean active)
+real_sync_allow_stop (NautilusWindow *window,
+		      NautilusWindowSlot *slot)
 {
 }
 
@@ -492,70 +435,121 @@ real_set_allow_up (NautilusWindow *window, gboolean allow)
 					      SPATIAL_ACTION_CLOSE_PARENT_FOLDERS);
 	gtk_action_set_sensitive (action, allow);
 
-	NAUTILUS_WINDOW_CLASS (parent_class)->set_allow_up (window, allow);
+	NAUTILUS_WINDOW_CLASS (nautilus_spatial_window_parent_class)->set_allow_up (window, allow);
+}
+
+static NautilusWindowSlot *
+real_open_slot (NautilusWindow *window,
+		NautilusWindowOpenSlotFlags flags)
+{
+	NautilusWindowSlot *slot;
+	GList *slots;
+
+	g_assert (nautilus_window_get_active_slot (window) == NULL);
+
+	slots = nautilus_window_get_slots (window);
+	g_assert (slots == NULL);
+	g_list_free (slots);
+
+	slot = g_object_new (NAUTILUS_TYPE_WINDOW_SLOT, NULL);
+	slot->window = window;
+	gtk_container_add (GTK_CONTAINER (NAUTILUS_SPATIAL_WINDOW (window)->details->content_box),
+			   slot->content_box);
+	gtk_widget_show (slot->content_box);
+	return slot;
+}
+
+static void
+real_close_slot (NautilusWindow *window,
+		 NautilusWindowSlot *slot)
+{
+	g_assert (g_list_length (window->details->slots) == 1);
+
+	nautilus_spatial_window_save_geometry (slot);
+	nautilus_spatial_window_save_scroll_position (slot);
+	nautilus_spatial_window_save_show_hidden_files_mode (slot);
+
+	EEL_CALL_PARENT (NAUTILUS_WINDOW_CLASS,
+			 close_slot, (window, slot));
 }
 
 static void
 location_menu_item_activated_callback (GtkWidget *menu_item,
-				       NautilusSpatialWindow *window)
+				       NautilusWindow *window)
 {
-	GnomeVFSURI *uri;
+	NautilusWindowSlot *slot;
 	char *location;
+	GFile *current;
+	GFile *dest;
 	GdkEvent *event;
 
-	uri = g_object_get_data (G_OBJECT (menu_item), "uri");
-	location = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+	slot = window->details->active_slot;
+
+	location = nautilus_window_slot_get_location_uri (slot);
+	current = g_file_new_for_uri (location);
+	g_free (location);
+
+	dest = g_object_get_data (G_OBJECT (menu_item), "uri");
+
 	event = gtk_get_current_event();
 
-	if (!gnome_vfs_uri_equal (uri, window->details->location))
+	if (!g_file_equal (current, dest))
 	{
+		GFile *child;
 		gboolean close_behind;
 		GList *selection;
-		GnomeVFSURI *child_uri;
 
 		close_behind = FALSE;
 		selection = NULL;
-		child_uri = g_object_get_data (G_OBJECT(menu_item), "child_uri");
-		if (child_uri != NULL) {
-			char *child_location;
-			child_location = gnome_vfs_uri_to_string (child_uri, GNOME_VFS_URI_HIDE_NONE);
-			selection = g_list_prepend (NULL, child_location);
+
+		child = g_object_get_data (G_OBJECT(menu_item), "child_uri");
+		if (child != NULL) {
+			selection = g_list_prepend (NULL, g_object_ref (child));
 		}
+
 		if (event != NULL && ((GdkEventAny *) event)->type == GDK_BUTTON_RELEASE &&
 		   (((GdkEventButton *) event)->button == 2 ||
 		   (((GdkEventButton *) event)->state & GDK_SHIFT_MASK) != 0))
 		{
 			close_behind = TRUE;
 		}
-		nautilus_window_open_location_with_selection (NAUTILUS_WINDOW (window), location, selection, close_behind);
+
+		nautilus_window_slot_open_location_with_selection
+			(slot, dest, selection, close_behind);
+
+		eel_g_object_list_free (selection);
 	}
 
 	if (event != NULL) {
 		gdk_event_free (event);
 	}
 
-	g_free (location);
-	
+	g_object_unref (current);
 }
 
 static void
 got_file_info_for_location_menu_callback (NautilusFile *file,
 					  gpointer callback_data)
 {	
-	GtkWidget *icon;
 	GtkWidget *menu_item = callback_data;
+	GtkWidget *label;
+	GtkWidget *icon;
 	GdkPixbuf *pixbuf;
-	char *icon_name;
+	char *name;
 
 	g_return_if_fail (NAUTILUS_IS_FILE (file));
 
 	pixbuf = NULL;
 
-	icon_name = nautilus_icon_factory_get_icon_for_file (file, FALSE);
-	if (icon_name != NULL) {
-		pixbuf = nautilus_icon_factory_get_pixbuf_from_name_with_stock_size (icon_name, NULL, GTK_ICON_SIZE_MENU, NULL);
-		g_free (icon_name);
-	}
+	name = nautilus_file_get_display_name (file);
+	label = gtk_bin_get_child (GTK_BIN (menu_item));
+	gtk_label_set_label (GTK_LABEL (label), name);
+	g_free (name);
+
+	pixbuf = nautilus_file_get_icon_pixbuf (file,
+						nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU),
+						TRUE,
+						NAUTILUS_FILE_ICON_FLAGS_IGNORE_VISITING);
 
 	if (pixbuf != NULL) {
 		icon = gtk_image_new_from_pixbuf (pixbuf);
@@ -613,62 +607,93 @@ location_button_pressed_callback (GtkWidget      *widget,
 				  GdkEventButton *event,
 				  NautilusWindow *window)
 {
-	if (event->button == 3 &&
-	    window->content_view != NULL) {
-		nautilus_view_pop_up_location_context_menu (window->content_view, event);
+	NautilusView *view;
+
+	view = window->details->active_slot->content_view;
+
+	if (event->button == 3 && view != NULL) {
+		nautilus_view_pop_up_location_context_menu (view, event, NULL);
 	}
 
 	return FALSE;
 }
 
 static void
-location_button_clicked_callback (GtkWidget *widget, NautilusSpatialWindow *window)
+location_button_clicked_callback (GtkWidget             *widget,
+				  NautilusSpatialWindow *window)
 {
+	NautilusWindowSlot *slot;
 	GtkWidget *popup, *menu_item, *first_item = NULL;
-	GnomeVFSURI *uri;
-	GnomeVFSURI *child_uri;
-	char *name;
+	char *location;
+	GFile *uri;
+	GFile *child_uri;
 	GMainLoop *loop;
 
-	
-	g_return_if_fail (window->details->location != NULL);
+	slot = NAUTILUS_WINDOW (window)->details->active_slot;
 
 	popup = gtk_menu_new ();
 	first_item = NULL;
-	uri = gnome_vfs_uri_ref (window->details->location);
+
+	location = nautilus_window_slot_get_location_uri (slot);
+	g_return_if_fail (location != NULL);
+
+	uri = g_file_new_for_uri (location);
+	g_free (location);
+
 	child_uri = NULL;
 	while (uri != NULL) {
 		NautilusFile *file;
-		char *uri_string;
+		char *name;
 
-		name = nautilus_get_uri_shortname_for_display (uri);
+		file = nautilus_file_get (uri);
+
+		name = nautilus_file_get_display_name (file);
 		menu_item = gtk_image_menu_item_new_with_label (name);
 		g_free (name);
+
 		if (first_item == NULL) {
 			first_item = menu_item;
 		}
-		
-		uri_string = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
-		file = nautilus_file_get (uri_string);
+
 		g_object_ref (menu_item);
 		nautilus_file_call_when_ready (file,
-					       NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY,
+					       NAUTILUS_FILE_ATTRIBUTE_INFO,
 					       got_file_info_for_location_menu_callback,
 					       menu_item);
-		g_free (uri_string);		
 
 		gtk_widget_show (menu_item);
 		g_signal_connect (menu_item, "activate",
 				  G_CALLBACK (location_menu_item_activated_callback),
 				  window);
-		g_object_set_data_full (G_OBJECT (menu_item), "uri", uri, (GDestroyNotify)gnome_vfs_uri_unref);
-		g_object_set_data_full (G_OBJECT (menu_item), "child_uri", child_uri, (GDestroyNotify)gnome_vfs_uri_unref);
+
+		g_object_set_data_full (G_OBJECT (menu_item),
+					"uri",
+					g_object_ref (uri),
+					(GDestroyNotify)g_object_unref);
+
+		if (child_uri) {
+			g_object_set_data_full (G_OBJECT (menu_item),
+						"child_uri",
+						g_object_ref (child_uri),
+						(GDestroyNotify)g_object_unref);
+		}
 
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (popup), menu_item);
 
-		child_uri = gnome_vfs_uri_ref (uri);
-		uri = gnome_vfs_uri_get_parent (uri);
+		if (child_uri){
+			g_object_unref (child_uri);
+		}
+		child_uri = uri;
+		uri = g_file_get_parent (uri);
 	}
+
+	if (child_uri){
+		g_object_unref (child_uri);
+	}
+	if (uri){
+		g_object_unref (uri);
+	}
+
 	gtk_menu_set_screen (GTK_MENU (popup), gtk_widget_get_screen (widget));
 
 	loop = g_main_loop_new (NULL, FALSE);
@@ -690,14 +715,16 @@ static int
 get_dnd_icon_size (NautilusSpatialWindow *window)
 {
 	NautilusWindow *parent;
+	NautilusView *view;
 	NautilusZoomLevel zoom_level;
 
 	parent = NAUTILUS_WINDOW(window);
+	view = parent->details->active_slot->content_view;
 
-	if (parent->content_view == NULL) {
+	if (view == NULL) {
 		return NAUTILUS_ICON_SIZE_STANDARD;
 	} else {
-		zoom_level = nautilus_view_get_zoom_level (parent->content_view);
+		zoom_level = nautilus_view_get_zoom_level (view);
 		return nautilus_get_icon_size_for_zoom_level (zoom_level);
 	}
 	
@@ -708,10 +735,15 @@ location_button_drag_begin_callback (GtkWidget             *widget,
 				     GdkDragContext        *context,
 				     NautilusSpatialWindow *window)
 {
+	NautilusWindowSlot *slot;
 	GdkPixbuf *pixbuf;
 
-	pixbuf = nautilus_icon_factory_get_pixbuf_for_file (NAUTILUS_WINDOW (window)->details->viewed_file,
-							    "open", get_dnd_icon_size (window), FALSE);
+	slot = NAUTILUS_WINDOW (window)->details->active_slot;
+
+	pixbuf = nautilus_file_get_icon_pixbuf (slot->viewed_file,
+						get_dnd_icon_size (window),
+						FALSE,
+						NAUTILUS_FILE_ICON_FLAGS_IGNORE_VISITING | NAUTILUS_FILE_ICON_FLAGS_FOR_DRAG_ACCEPT);
 
 	gtk_drag_set_icon_pixbuf (context, pixbuf, 0, 0);
 
@@ -727,25 +759,26 @@ get_data_binder (NautilusDragEachSelectedItemDataGet iteratee,
 		 gpointer                            data)
 {
 	NautilusSpatialWindow *window;
+	NautilusWindowSlot *slot;
+	char *location;
 	int icon_size;
-	char *uri;
 
 	g_assert (NAUTILUS_IS_SPATIAL_WINDOW (iterator_context));
 	window = NAUTILUS_SPATIAL_WINDOW (iterator_context);
 
-	uri = gnome_vfs_uri_to_string (window->details->location,
-				       GNOME_VFS_URI_HIDE_NONE);
+	slot = NAUTILUS_WINDOW (window)->details->active_slot;
 
+	location = nautilus_window_slot_get_location_uri (slot);
 	icon_size = get_dnd_icon_size (window);
 
-	iteratee (uri,
+	iteratee (location,
 		  0,
 		  0,
 		  icon_size,
 		  icon_size,
 		  data);
 
-	g_free (uri);
+	g_free (location);
 }
 
 static void
@@ -762,38 +795,30 @@ location_button_drag_data_get_callback (GtkWidget             *widget,
 
 void
 nautilus_spatial_window_set_location_button  (NautilusSpatialWindow *window,
-					      const char            *location)
+					      GFile                 *location)
 {
-	GnomeVFSURI *uri;
-	char *name;
-	
-	uri = NULL;
 	if (location != NULL) {
-		uri = gnome_vfs_uri_new (location);
-	}
-	if (uri != NULL) {
 		NautilusFile *file;
-		GnomeVFSResult vfs_result_code;
-		
-		name = nautilus_get_uri_shortname_for_display (uri);
+		char *name;
+		GError *error;
+
+		file = nautilus_file_get (location);
+
+		/* FIXME: monitor for name change... */
+		name = nautilus_file_get_display_name (file);
 		gtk_label_set_label (GTK_LABEL (window->details->location_label),
 				     name);
 		g_free (name);
 		gtk_widget_set_sensitive (window->details->location_button, TRUE);
 
-		file = nautilus_file_get (location);
-		vfs_result_code = nautilus_file_get_file_info_result (file);
-		if (vfs_result_code == GNOME_VFS_OK) {
-			char *icon_name;
+		error = nautilus_file_get_file_info_error (file);
+		if (error == NULL) {
 			GdkPixbuf *pixbuf;
 
-			pixbuf = NULL;
-
-			icon_name = nautilus_icon_factory_get_icon_for_file (file, FALSE);		
-			if (icon_name != NULL) {
-				pixbuf = nautilus_icon_factory_get_pixbuf_from_name_with_stock_size (icon_name, NULL, GTK_ICON_SIZE_MENU, NULL);
-				g_free (icon_name);
-			}
+			pixbuf = nautilus_file_get_icon_pixbuf (file,
+								nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU),
+								TRUE,
+								NAUTILUS_FILE_ICON_FLAGS_IGNORE_VISITING);
 
 			if (pixbuf != NULL) {
 				gtk_image_set_from_pixbuf (GTK_IMAGE (window->details->location_icon),  pixbuf);
@@ -810,11 +835,6 @@ nautilus_spatial_window_set_location_button  (NautilusSpatialWindow *window,
 				     "");
 		gtk_widget_set_sensitive (window->details->location_button, FALSE);
 	}
-
-	if (window->details->location != NULL) {
-		gnome_vfs_uri_unref (window->details->location);
-	}
-	window->details->location = uri;
 }
 
 static void
@@ -854,48 +874,54 @@ action_search_callback (GtkAction *action,
 {
 	NautilusWindow *window;
 	char *uri;
+	GFile *f;
 
 	window = NAUTILUS_WINDOW (user_data);
 
 	uri = nautilus_search_directory_generate_new_uri ();
-	nautilus_window_go_to (window, uri);
+	f = g_file_new_for_uri (uri);
+	nautilus_window_go_to (window, f);
+	g_object_unref (f);
 	g_free (uri);
 }
 
 static const GtkActionEntry spatial_entries[] = {
-  { SPATIAL_ACTION_PLACES, NULL, N_("_Places") },               /* name, stock id, label */
-  { SPATIAL_ACTION_GO_TO_LOCATION, NULL, N_("Open _Location..."), /* name, stock id, label */
-    "<control>L", N_("Specify a location to open"),
-    G_CALLBACK (action_go_to_location_callback) },
-  { SPATIAL_ACTION_CLOSE_PARENT_FOLDERS, NULL, N_("Close P_arent Folders"), /* name, stock id, label */
-    "<control><shift>W", N_("Close this folder's parents"),
-    G_CALLBACK (action_close_parent_folders_callback) },
-  { SPATIAL_ACTION_CLOSE_ALL_FOLDERS, NULL, N_("Clos_e All Folders"), /* name, stock id, label */
-    "<control>Q", N_("Close all folder windows"),
-    G_CALLBACK (action_close_all_folders_callback) },
-  { "Add Bookmark", GTK_STOCK_ADD, N_("_Add Bookmark"), /* name, stock id, label */
-    "<control>d", N_("Add a bookmark for the current location to this menu"),
-    G_CALLBACK (action_add_bookmark_callback) },
-  { "Edit Bookmarks", NULL, N_("_Edit Bookmarks"), /* name, stock id, label */
-    "<control>b", N_("Display a window that allows editing the bookmarks in this menu"),
-    G_CALLBACK (action_edit_bookmarks_callback) },
-  { "Search", "gtk-find", N_("_Search for Files..."), /* name, stock id, label */
-    "<control>F", N_("Locate documents and folders on this computer by name or content"),
-    G_CALLBACK (action_search_callback) },
+  /* name, stock id, label */  { SPATIAL_ACTION_PLACES, NULL, N_("_Places") },
+  /* name, stock id, label */  { SPATIAL_ACTION_GO_TO_LOCATION, NULL, N_("Open _Location..."),
+                                 "<control>L", N_("Specify a location to open"),
+                                 G_CALLBACK (action_go_to_location_callback) },
+  /* name, stock id, label */  { SPATIAL_ACTION_CLOSE_PARENT_FOLDERS, NULL, N_("Close P_arent Folders"),
+                                 "<control><shift>W", N_("Close this folder's parents"),
+                                 G_CALLBACK (action_close_parent_folders_callback) },
+  /* name, stock id, label */  { SPATIAL_ACTION_CLOSE_ALL_FOLDERS, NULL, N_("Clos_e All Folders"), 
+                                 "<control>Q", N_("Close all folder windows"),
+                                 G_CALLBACK (action_close_all_folders_callback) },
+  /* name, stock id, label */  { "Add Bookmark", GTK_STOCK_ADD, N_("_Add Bookmark"),
+                                 "<control>d", N_("Add a bookmark for the current location to this menu"),
+                                 G_CALLBACK (action_add_bookmark_callback) },
+  /* name, stock id, label */  { "Edit Bookmarks", NULL, N_("_Edit Bookmarks..."),
+                                 "<control>b", N_("Display a window that allows editing the bookmarks in this menu"),
+                                 G_CALLBACK (action_edit_bookmarks_callback) },
+  /* name, stock id, label */  { "Search", "gtk-find", N_("_Search for Files..."),
+                                 "<control>F", N_("Locate documents and folders on this computer by name or content"),
+                                 G_CALLBACK (action_search_callback) },
 };
 
 static void
-nautilus_spatial_window_instance_init (NautilusSpatialWindow *window)
+nautilus_spatial_window_init (NautilusSpatialWindow *window)
 {
 	GtkRcStyle *rc_style;
 	GtkWidget *arrow;
-	GtkWidget *hbox, *vbox, *eventbox, *extras_vbox;
+	GtkWidget *hbox, *vbox;
 	GtkActionGroup *action_group;
 	GtkUIManager *ui_manager;
 	GtkTargetList *targets;
 	const char *ui;
-	
-	window->details = g_new0 (NautilusSpatialWindowDetails, 1);
+
+	window->details = G_TYPE_INSTANCE_GET_PRIVATE (window,
+						       NAUTILUS_TYPE_SPATIAL_WINDOW,
+						       NautilusSpatialWindowDetails);
+
 	window->affect_spatial_window_on_next_location_change = TRUE;
 
 	vbox = gtk_vbox_new (FALSE, 0);
@@ -906,21 +932,8 @@ nautilus_spatial_window_instance_init (NautilusSpatialWindow *window)
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  0,                                  0);
 	gtk_widget_show (vbox);
+	window->details->content_box = vbox;
 
-	eventbox = gtk_event_box_new ();
-	gtk_widget_set_name (eventbox, "nautilus-extra-view-widget");
-	gtk_box_pack_start (GTK_BOX (vbox), eventbox, FALSE, FALSE, 0);
-	gtk_widget_show (eventbox);
-	
-	extras_vbox = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (extras_vbox), 6);
-	NAUTILUS_WINDOW (window)->details->extra_location_widgets = extras_vbox;
-	gtk_container_add (GTK_CONTAINER (eventbox), extras_vbox);
-
-	window->details->content_box = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (vbox), window->details->content_box, TRUE, TRUE, 0);
-	gtk_widget_show (window->details->content_box);
-	
 	window->details->location_button = gtk_button_new ();
 	g_signal_connect (window->details->location_button,
 			  "button-press-event",
@@ -1014,23 +1027,20 @@ nautilus_spatial_window_class_init (NautilusSpatialWindowClass *class)
 
 	NAUTILUS_WINDOW_CLASS (class)->prompt_for_location = 
 		real_prompt_for_location;
-	NAUTILUS_WINDOW_CLASS (class)->set_search_mode = 
-		real_set_search_mode;
-	NAUTILUS_WINDOW_CLASS (class)->get_icon_name =
-		real_get_icon_name;
-	NAUTILUS_WINDOW_CLASS (class)->set_title = 
-		real_set_title;
-	NAUTILUS_WINDOW_CLASS (class)->set_content_view_widget = 
-		real_set_content_view_widget;
-	NAUTILUS_WINDOW_CLASS (class)->close = 
-		real_window_close;
+	NAUTILUS_WINDOW_CLASS (class)->get_icon =
+		real_get_icon;
+	NAUTILUS_WINDOW_CLASS (class)->sync_title = 
+		real_sync_title;
 	NAUTILUS_WINDOW_CLASS(class)->get_default_size = real_get_default_size;
 
-	NAUTILUS_WINDOW_CLASS(class)->set_throbber_active =
-		real_set_throbber_active;
+	NAUTILUS_WINDOW_CLASS(class)->sync_allow_stop =
+		real_sync_allow_stop;
 	NAUTILUS_WINDOW_CLASS(class)->set_allow_up =
 		real_set_allow_up;
 
+	NAUTILUS_WINDOW_CLASS (class)->open_slot = real_open_slot;
+	NAUTILUS_WINDOW_CLASS (class)->close_slot = real_close_slot;
+
 	binding_set = gtk_binding_set_by_class (class);
 	gtk_binding_entry_add_signal (binding_set, GDK_BackSpace, GDK_SHIFT_MASK,
 				      "go_up", 1,
@@ -1038,5 +1048,6 @@ nautilus_spatial_window_class_init (NautilusSpatialWindowClass *class)
 	gtk_binding_entry_add_signal (binding_set, GDK_Up, GDK_SHIFT_MASK | GDK_MOD1_MASK,
 				      "go_up", 1,
 				      G_TYPE_BOOLEAN, TRUE);
-	
+
+	g_type_class_add_private (G_OBJECT_CLASS (class), sizeof(NautilusSpatialWindowDetails));
 }
